"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HELIX_ENDPOINTS = {
    helix: [{
            default: 'http://localhost:8100/admin/v2'
        }]
};
exports.SESSION_STORE = undefined;
exports.SSL = {
    port: 0,
    keyfile: '',
    certfile: '',
    passfile: '',
    cafiles: []
};
exports.LDAP = {
    uri: 'ldap://example.com',
    base: 'DC=example,DC=com',
    principalSuffix: '@example.com',
    adminGroup: 'admin'
};
//# sourceMappingURL=config.js.map
